/* Copyright 2013, Unpublished Work of Technologic Systems                 
 * All Rights Reserved.                                                    
 *                                                                         
 * THIS WORK IS AN UNPUBLISHED WORK AND CONTAINS CONFIDENTIAL,             
 * PROPRIETARY AND TRADE SECRET INFORMATION OF TECHNOLOGIC SYSTEMS.        
 * ACCESS TO THIS WORK IS RESTRICTED TO (I) TECHNOLOGIC SYSTEMS            
 * EMPLOYEES WHO HAVE A NEED TO KNOW TO PERFORM TASKS WITHIN THE SCOPE     
 * OF THEIR ASSIGNMENTS AND (II) ENTITIES OTHER THAN TECHNOLOGIC           
 * SYSTEMS WHO HAVE ENTERED INTO APPROPRIATE LICENSE AGREEMENTS.  NO       
 * PART OF THIS WORK MAY BE USED, PRACTICED, PERFORMED, COPIED,            
 * DISTRIBUTED, REVISED, MODIFIED, TRANSLATED, ABRIDGED, CONDENSED,        
 * EXPANDED, COLLECTED, COMPILED, LINKED, RECAST, TRANSFORMED, ADAPTED     
 * IN ANY FORM OR BY ANY MEANS, MANUAL, MECHANICAL, CHEMICAL,              
 * ELECTRICAL, ELECTRONIC, OPTICAL, BIOLOGICAL, OR OTHERWISE WITHOUT       
 * THE PRIOR WRITTEN PERMISSION AND CONSENT OF TECHNOLOGIC SYSTEMS.        
 * ANY USE OR EXPLOITATION OF THIS WORK WITHOUT THE PRIOR WRITTEN          
 * CONSENT OF TECHNOLOGIC SYSTEMS COULD SUBJECT THE PERPETRATOR TO         
 * CRIMINAL AND CIVIL LIABILITY.                                           
 */                                      
 
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include "nbus-spi.h"


#ifdef DEBUG
#define DEBUGMSG(msg,...)\
  fprintf(stderr,msg,__VA_ARGS__);
#else
#define DEBUGMSG(msg,...)
#endif

#define SPI_BASE   0x60

unsigned short spiR0, spiR0valid;
static int maxspeed;
static unsigned last_freq,last_edge;

static int spi_f[] = {
  75000000,37500000,18750000,12500000,9375000,
  7500000,6250000,5360000,4680000,4170000,
  3750000,3409090,3125000,2884615,2678571,
  2500000,2343750,2205882,2083333,1973684,
  1875000,1785714,1704545,1630434,1562500,
  1500000,1442307,1388888,1339285,1293103,
  1250000,0 // 1209677
};

static unsigned short getR0(int force) {
  if (spiR0valid && !force) return spiR0;
  spiR0 = nbus_peek16(SPI_BASE);
  spiR0valid = 1;
  return spiR0;
}

static void setR0(unsigned short val) {
  if (spiR0valid && spiR0 == val) return;
  spiR0valid = 1;
  spiR0 = val;
  nbus_poke16(SPI_BASE,val);  
}



void nbus_spi_init(void) {
   unsigned short x;
   spiR0 = spiR0valid = last_freq = last_edge = 0;  
   maxspeed = 0;        
   x = nbus_peek16(0x06);
   nbus_poke16(0x06, x | 0x10);   
}

int nbus_spi_lun(int n) {
  if (n < 0 || n > 3) return 3 & (getR0(0) >> 8);
  setR0((getR0(0) & (~(3<<8))) | (n << 8));
  return n;
}



void nbus_spi_write(int octets, unsigned char *buf,int de_cs) {
   
  int n = octets;
  while (n > 1) {
    nbus_poke16(SPI_BASE+0x8,(unsigned short)*buf++);    
    n--;
  }   
  if (n > 0)
     nbus_poke16(de_cs?(SPI_BASE+0x0C):(SPI_BASE+0x8),(unsigned short)*buf);
}


void nbus_spi_speed(unsigned freq,int edge) {
  int i=0;
  unsigned short val;

  val = getR0(0);
  if (freq > 0) {
    last_freq = freq;
    while (spi_f[i] > freq) i++;
    DEBUGMSG("nbus_spi_speed(%d): [%d]=%d\n",freq,i,spi_f[i]);
    maxspeed = (i == 0);
    val &= ~(15<<10); // mask out existing speed[3:0] bits
    val |= (i& 15) << 10; // or in new speed[3:0] bits
    val &= ~(1 << 0); // mask out existing speed[4] bit
    val |= ((i >> 4) & 1); // or in new speed[4] bit
  }
  if (edge != 0) {
    last_edge = edge;
    val &= ~(1<<14); // mask out existing clock polarity bit
    DEBUGMSG("also: %d,%d",edge>0,maxspeed);
    val |= ((edge>0||maxspeed)?0:1) << 14; // or in new clock polarity bit
  } 
  setR0(val);
}




